@@ -1,12 +1,18 @@
 #!/usr/bin/env python
 
+
+
 import os, sys, traceback
-import Queue
 import getpass
 from threading import Thread
-from string import find, split, join
 from subprocess import *
 
+if(sys.hexversion < 0x03000000):
+	import Queue
+else:
+	import queue as Queue
+
+
 # svmtrain and gnuplot executable
 
 is_win32 = (sys.platform == 'win32')
@@ -47,7 +53,7 @@ Usage: grid.py [-log2c begin,end,step] [-log2g begin,end,step] [-v fold]
 [additional parameters for svm-train] dataset"""
 
     if len(argv) < 2:
-        print usage
+        print(usage)
         sys.exit(1)
 
     dataset_pathname = argv[-1]
@@ -60,16 +66,16 @@ Usage: grid.py [-log2c begin,end,step] [-log2g begin,end,step] [-v fold]
     while i < len(argv) - 1:
         if argv[i] == "-log2c":
             i = i + 1
-            (c_begin,c_end,c_step) = map(float,split(argv[i],","))
+            (c_begin,c_end,c_step) = map(float,argv[i].split(","))
         elif argv[i] == "-log2g":
             i = i + 1
-            (g_begin,g_end,g_step) = map(float,split(argv[i],","))
+            (g_begin,g_end,g_step) = map(float,argv[i].split(","))
         elif argv[i] == "-v":
             i = i + 1
             fold = argv[i]
         elif argv[i] in ('-c','-g'):
-            print "Option -c and -g are renamed."
-            print usage
+            print("Option -c and -g are renamed.")
+            print(usage)
             sys.exit(1)
         elif argv[i] == '-svmtrain':
             i = i + 1
@@ -87,7 +93,7 @@ Usage: grid.py [-log2c begin,end,step] [-log2g begin,end,step] [-v fold]
             pass_through_options.append(argv[i])
         i = i + 1
 
-    pass_through_string = join(pass_through_options," ")
+    pass_through_string = " ".join(pass_through_options)
     assert os.path.exists(svmtrain_exe),"svm-train executable not found"    
     assert os.path.exists(gnuplot_exe),"gnuplot executable not found"
     assert os.path.exists(dataset_pathname),"dataset not found"
@@ -121,52 +127,51 @@ def permute_sequence(seq):
 
 def redraw(db,best_param,tofile=False):
     if len(db) == 0: return
-    begin_level = round(max(map(lambda(x):x[2],db))) - 3
+    begin_level = round(max(x[2] for x in db)) - 3
     step_size = 0.5
 
     best_log2c,best_log2g,best_rate = best_param
 
     if tofile:
-        gnuplot.write("set term png transparent small\n")
-        gnuplot.write("set output \"%s\"\n" % png_filename.replace('\\','\\\\'))
-        #gnuplot.write("set term postscript color solid\n")
-        #gnuplot.write("set output \"%s.ps\"\n" % dataset_title)
+        gnuplot.write( "set term png transparent small\n".encode())
+        gnuplot.write( ("set output \"%s\"\n" % png_filename.replace('\\','\\\\')).encode())
+        #gnuplot.write("set term postscript color solid\n".encode())
+        #gnuplot.write(("set output \"%s.ps\"\n" % dataset_title).encode())
     elif is_win32:
-        gnuplot.write("set term windows\n")
+        gnuplot.write("set term windows\n".encode())
     else:
-        gnuplot.write("set term x11\n")
-    gnuplot.write("set xlabel \"log2(C)\"\n")
-    gnuplot.write("set ylabel \"log2(gamma)\"\n")
-    gnuplot.write("set xrange [%s:%s]\n" % (c_begin,c_end))
-    gnuplot.write("set yrange [%s:%s]\n" % (g_begin,g_end))
-    gnuplot.write("set contour\n")
-    gnuplot.write("set cntrparam levels incremental %s,%s,100\n" % (begin_level,step_size))
-    gnuplot.write("unset surface\n")
-    gnuplot.write("unset ztics\n")
-    gnuplot.write("set view 0,0\n")
-    gnuplot.write("set title \"%s\"\n" % dataset_title)
-    gnuplot.write("unset label\n")
-    gnuplot.write("set label \"Best log2(C) = %s  log2(gamma) = %s  accuracy = %s%%\" \
+        gnuplot.write( "set term x11\n".encode())
+    gnuplot.write("set xlabel \"log2(C)\"\n".encode())
+    gnuplot.write("set ylabel \"log2(gamma)\"\n".encode())
+    gnuplot.write(("set xrange [%s:%s]\n" % (c_begin,c_end)).encode())
+    gnuplot.write(("set yrange [%s:%s]\n" % (g_begin,g_end)).encode())
+    gnuplot.write("set contour\n".encode())
+    gnuplot.write(("set cntrparam levels incremental %s,%s,100\n" % (begin_level,step_size)).encode())
+    gnuplot.write("unset surface\n".encode())
+    gnuplot.write("unset ztics\n".encode())
+    gnuplot.write("set view 0,0\n".encode())
+    gnuplot.write(("set title \"%s\"\n" % dataset_title).encode())
+    gnuplot.write("unset label\n".encode())
+    gnuplot.write(("set label \"Best log2(C) = %s  log2(gamma) = %s  accuracy = %s%%\" \
                   at screen 0.5,0.85 center\n" % \
-                  (best_log2c, best_log2g, best_rate))
-    gnuplot.write("set label \"C = %s  gamma = %s\""
-                  " at screen 0.5,0.8 center\n" % (2**best_log2c, 2**best_log2g))
-    gnuplot.write("splot \"-\" with lines\n")
-    def cmp (x,y):
-        if x[0] < y[0]: return -1
-        if x[0] > y[0]: return 1
-        if x[1] > y[1]: return -1
-        if x[1] < y[1]: return 1
-        return 0
-    db.sort(cmp)
+                  (best_log2c, best_log2g, best_rate)).encode())
+    gnuplot.write(("set label \"C = %s  gamma = %s\""
+                  " at screen 0.5,0.8 center\n" % (2**best_log2c, 2**best_log2g)).encode())
+    gnuplot.write("splot \"-\" with lines\n".encode())
+    
+
+
+    
+    db.sort(key = lambda x:(x[0], -x[1]))
+
     prevc = db[0][0]
     for line in db:
         if prevc != line[0]:
-            gnuplot.write("\n")
+            gnuplot.write("\n".encode())
             prevc = line[0]
-        gnuplot.write("%s %s %s\n" % line)
-    gnuplot.write("e\n")
-    gnuplot.write("\n") # force gnuplot back to prompt when term set failure
+        gnuplot.write(("%s %s %s\n" % line).encode())
+    gnuplot.write("e\n".encode())
+    gnuplot.write("\n".encode()) # force gnuplot back to prompt when term set failure
     gnuplot.flush()
 
 
@@ -217,9 +222,11 @@ class Worker(Thread):
                 if rate is None: raise "get no rate"
             except:
                 # we failed, let others do that and we just quit
-                traceback.print_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)
+            
+                traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
+                
                 self.job_queue.put((cexp,gexp))
-                print 'worker %s quit.' % self.name
+                print('worker %s quit.' % self.name)
                 break
             else:
                 self.result_queue.put((self.name,cexp,gexp,rate))
@@ -230,8 +237,8 @@ class LocalWorker(Worker):
           (svmtrain_exe,c,g,fold,pass_through_string,dataset_pathname)
         result = Popen(cmdline,shell=True,stdout=PIPE).stdout
         for line in result.readlines():
-            if find(line,"Cross") != -1:
-                return float(split(line)[-1][0:-1])
+            if str(line).find("Cross") != -1:
+                return float(line.split()[-1][0:-1])
 
 class SSHWorker(Worker):
     def __init__(self,name,job_queue,result_queue,host):
@@ -244,8 +251,8 @@ class SSHWorker(Worker):
            svmtrain_exe,c,g,fold,pass_through_string,dataset_pathname)
         result = Popen(cmdline,shell=True,stdout=PIPE).stdout
         for line in result.readlines():
-            if find(line,"Cross") != -1:
-                return float(split(line)[-1][0:-1])
+            if str(line).find("Cross") != -1:
+                return float(line.split()[-1][0:-1])
 
 class TelnetWorker(Worker):
     def __init__(self,name,job_queue,result_queue,host,username,password):
@@ -264,7 +271,7 @@ class TelnetWorker(Worker):
         # XXX: how to know whether login is successful?
         tn.read_until(self.username)
         # 
-        print 'login ok', self.host
+        print('login ok', self.host)
         tn.write("cd "+os.getcwd()+"\n")
         Worker.run(self)
         tn.write("exit\n")               
@@ -273,9 +280,9 @@ class TelnetWorker(Worker):
           (svmtrain_exe,c,g,fold,pass_through_string,dataset_pathname)
         result = self.tn.write(cmdline+'\n')
         (idx,matchm,output) = self.tn.expect(['Cross.*\n'])
-        for line in split(output,'\n'):
-            if find(line,"Cross") != -1:
-                return float(split(line)[-1][0:-1])
+        for line in output.split('\n'):
+            if str(line).find("Cross") != -1:
+                return float(line.split()[-1][0:-1])
 
 def main():
 
@@ -293,21 +300,8 @@ def main():
         for (c,g) in line:
             job_queue.put((c,g))
 
-    # hack the queue to become a stack --
-    # this is important when some thread
-    # failed and re-put a job. If we still
-    # use FIFO, the job will be put
-    # into the end of the queue, and the graph
-    # will only be updated in the end
+    job_queue._put = job_queue.queue.appendleft
 
-    def _put(self,item):
-        if sys.hexversion >= 0x020400A1:
-            self.queue.appendleft(item)
-        else:
-            self.queue.insert(0,item)
-
-    import new
-    job_queue._put = new.instancemethod(_put,job_queue,job_queue.__class__)
 
     # fire telnet workers
 
@@ -333,30 +327,33 @@ def main():
     # gather results
 
     done_jobs = {}
-    result_file = open(out_filename,'w',0)
+
+
+    result_file = open(out_filename, 'w')
+
+
     db = []
     best_rate = -1
     best_c1,best_g1 = None,None
 
     for line in jobs:
         for (c,g) in line:
-            while not done_jobs.has_key((c,g)):
+            while (c, g) not in done_jobs:
                 (worker,c1,g1,rate) = result_queue.get()
                 done_jobs[(c1,g1)] = rate
                 result_file.write('%s %s %s\n' %(c1,g1,rate))
                 result_file.flush()
-                print "[%s] %s %s %s" % (worker,c1,g1,rate),
                 if (rate > best_rate) or (rate==best_rate and g1==best_g1 and c1<best_c1):
                     best_rate = rate
                     best_c1,best_g1=c1,g1
                     best_c = 2.0**c1
                     best_g = 2.0**g1
-                print " (best c=%s, g=%s, rate=%s)" % \
-                    (best_c, best_g, best_rate)
+                print("[%s] %s %s %s (best c=%s, g=%s, rate=%s)" % \
+		    (worker,c1,g1,rate, best_c, best_g, best_rate))
             db.append((c,g,done_jobs[(c,g)]))
         redraw(db,[best_c1, best_g1, best_rate])
         redraw(db,[best_c1, best_g1, best_rate],True)
 
     job_queue.put((WorkerStopToken,None))
-    print "%s %s %s" % (best_c, best_g, best_rate)
+    print("%s %s %s" % (best_c, best_g, best_rate))
 main()
